// src/cairo_widget.hh - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef CAIRO_WIDGET_HH
#define CAIRO_WIDGET_HH 1

#include "config.h"

#include "backend/history_observer.hh"

#include <cairo.h>
#include <gtkmm/widget.h>
#include <gdkmm/pixmap.h>

namespace sgpem
{
  class CairoWidget : public Gtk::Widget, public HistoryObserver
  {
  public:
    CairoWidget();
    virtual ~CairoWidget();

    void update(const History& history);

  protected:
    typedef unsigned int    size_t;

    virtual void on_realize();
    virtual void on_size_allocate(const Gtk::Allocation& allocation);
    virtual bool on_expose_event(GdkEventExpose* event);
    virtual bool on_button_press_event(GdkEventButton* event);
    virtual void draw_widget(cairo_t* ctx) = 0;
    virtual void calc_size(const History& history, size_t& width, size_t& height) const = 0;

    typedef Gdk::Rectangle                           area_t;
    typedef sigc::mem_functor0<void, CairoWidget>    method0_t;
    typedef std::pair<area_t, method0_t>             area_callback_t;
    typedef std::vector<area_callback_t>             areas_vect_t;

  private:
    // The width and height the widget will assume, must be determined
    // before starting drawing by calc_size()
    mutable size_t      _w;
    mutable size_t      _h;

    // The offscreen pixmap we use for double-buffering
    Glib::RefPtr<Gdk::Pixmap>  _buf;
  };

} //~ namespace sgpem


#endif //~ CAIRO_WIDGET_HH
